#ifndef NPSTAT_SUMOFSQUARES_HH_
#define NPSTAT_SUMOFSQUARES_HH_

/*!
// \file sumOfSquares.hh
//
// \brief A simple code to calculate the sum of squares of elements
//        inside a contiguous buffer
//
// Author: I. Volobouev
//
// November 2020
*/

#include <cassert>

#include "npstat/nm/PreciseType.hh"

namespace npstat {
    template<typename Numeric>
    typename PreciseType<Numeric>::type sumOfSquares(
        const Numeric* data, const unsigned long len)
    {
        typedef typename PreciseType<Numeric>::type Precise;
        Precise sum = Precise();
        if (len)
        {
            assert(data);
            for (unsigned long i=0; i<len; ++i)
            {
                const Precise v = static_cast<Precise>(data[i]);
                sum += v*v;
            }
        }
        return sum;
    }
}

#endif // NPSTAT_SUMOFSQUARES_HH_
