#ifndef NPSTAT_BANDWIDTHCVPSEUDOLOGLIND_HH_
#define NPSTAT_BANDWIDTHCVPSEUDOLOGLIND_HH_

/*!
// \file BandwidthCVPseudoLogliND.hh
//
// \brief Cross-validating multivariate density estimates by optimizing
//        regularized pseudo log-likelihood
//
// Author: I. Volobouev
//
// September 2010
*/

#include "npstat/stat/AbsBandwidthCV.hh"

namespace npstat {
    /**
    // Class for calculating KDE or LOrPE cross-validation pseudo log
    // likelihood, for multivariate density estimates
    */
    template<typename Num, class Array>
    class BandwidthCVPseudoLogliND : public AbsBandwidthCVND<Num,Array>
    {
    public:
        /**
        // Parameter "regularizationPower" is used to limit the contributions
        // into the overall pseudo log likelihood from points for which the
        // "leaving one out" density is very low. For those points, instead
        // of "leaving one out" density, we will use the density generated by
        // that point itself divided by pow(N, regularizationPower). This
        // method limits the effect of tails on bandwidth determination.
        */
        explicit BandwidthCVPseudoLogliND(double regularizationPower=0.5)
            : renormPow_(regularizationPower), nonZeroCount_(0), renormCount_(0) {}

        inline virtual ~BandwidthCVPseudoLogliND() {}

        inline unsigned long getNonZeroCount() const {return nonZeroCount_;}
        inline unsigned long getRenormCount() const {return renormCount_;}

        // Unweighted samples
        virtual double operator()(
            const HistoND<Num>& histo,
            const Array& densityEstimate,
            const AbsPolyFilterND& filterUsed) const;

        // Weighted samples
        virtual double operator()(
            const HistoND<Num>& histo,
            double effectiveSampleSize,
            const Array& densityEstimate,
            const AbsPolyFilterND& filterUsed) const;

    private:
        double renormPow_;
        mutable unsigned long nonZeroCount_;
        mutable unsigned long renormCount_;
    };
}

#include "npstat/stat/BandwidthCVPseudoLogliND.icc"

#endif // NPSTAT_BANDWIDTHCVPSEUDOLOGLIND_HH_
