#include "npstat/stat/Distribution1DFactory.hh"
#include "npstat/stat/Distributions1D.hh"
#include "npstat/stat/JohnsonCurves.hh"
#include "npstat/stat/GaussianMixture1D.hh"
#include "npstat/stat/QuantileTable1D.hh"
#include "npstat/stat/UGaussConvolution1D.hh"
#include "npstat/stat/DeltaMixture1D.hh"
#include "npstat/stat/PolynomialDistro1D.hh"

namespace npstat {
    DefaultScalableDistribution1DFactory::
    DefaultScalableDistribution1DFactory()
    {
        (*this)["Beta1D"] =
            new ScalableDistribution1DFactory<Beta1D>();
        (*this)["BifurcatedGauss1D"] =
            new ScalableDistribution1DFactory<BifurcatedGauss1D>();
        (*this)["BinnedDensity1D"] =
            new ScalableDistribution1DFactory<BinnedDensity1D>();
        (*this)["Cauchy1D"] = new ScalableDistribution1DFactory<Cauchy1D>();
        (*this)["DeltaMixture1D"] = new ScalableDistribution1DFactory<DeltaMixture1D>();
        (*this)["Exponential1D"] =
            new ScalableDistribution1DFactory<Exponential1D>();
        (*this)["Gamma1D"] =
            new ScalableDistribution1DFactory<Gamma1D>();
        (*this)["Gauss1D"] = new ScalableDistribution1DFactory<Gauss1D>();
        (*this)["GaussianMixture1D"] =
            new ScalableDistribution1DFactory<GaussianMixture1D>();
        (*this)["Huber1D"] = new ScalableDistribution1DFactory<Huber1D>();
        (*this)["IsoscelesTriangle1D"] =
            new ScalableDistribution1DFactory<IsoscelesTriangle1D>();
        (*this)["JohnsonSb"] = new ScalableDistribution1DFactory<JohnsonSb>();
        (*this)["JohnsonSu"] = new ScalableDistribution1DFactory<JohnsonSu>();
        (*this)["JohnsonSystem"] =
            new ScalableDistribution1DFactory<JohnsonSystem>();
        //
        // Even though LocationScaleFamily1D class inherits from 
        // ScalableDistribution1D, it can not be constructed from
        // the parameter list alone. Threfore, the entry for
        // "LocationScaleFamily1D" factory is missing.
        //
        (*this)["Logistic1D"] = new ScalableDistribution1DFactory<Logistic1D>();
        (*this)["LogNormal"] = new ScalableDistribution1DFactory<LogNormal>();
        (*this)["LogQuadratic1D"] =
            new ScalableDistribution1DFactory<LogQuadratic1D>();
        (*this)["MirroredGauss1D"] =
            new ScalableDistribution1DFactory<MirroredGauss1D>();
        (*this)["Moyal1D"] = new ScalableDistribution1DFactory<Moyal1D>();
        (*this)["Pareto1D"] = new ScalableDistribution1DFactory<Pareto1D>();
        (*this)["Quadratic1D"] =
            new ScalableDistribution1DFactory<Quadratic1D>();
        (*this)["QuantileTable1D"] =
            new ScalableDistribution1DFactory<QuantileTable1D>();
        (*this)["StudentsT1D"] =
            new ScalableDistribution1DFactory<StudentsT1D>();
        (*this)["SymmetricBeta1D"] =
            new ScalableDistribution1DFactory<SymmetricBeta1D>();
        (*this)["Tabulated1D"] =
            new ScalableDistribution1DFactory<Tabulated1D>();
        (*this)["TruncatedGauss1D"] =
            new ScalableDistribution1DFactory<TruncatedGauss1D>();
        (*this)["UGaussConvolution1D"] =
            new ScalableDistribution1DFactory<UGaussConvolution1D>();
        (*this)["Uniform1D"] = new ScalableDistribution1DFactory<Uniform1D>();
        (*this)["UniPareto1D"] = new ScalableDistribution1DFactory<UniPareto1D>();
        (*this)["PolynomialDistro1D"] = new ScalableDistribution1DFactory<PolynomialDistro1D>();
    }

    DefaultScalableDistribution1DFactory::
    ~DefaultScalableDistribution1DFactory()
    {
        for (std::map<std::string, AbsScalableDistribution1DFactory*>::
                 iterator it = begin(); it != end(); ++it)
            delete it->second;
    }
}
