#include <cassert>
#include <cstring>
#include <sstream>
#include <stdexcept>

#include "npstat/stat/EdgeworthSeriesMethod.hh"

static const char* names[] = {
    "EDGEWORTH_SEVERINI",
    "EDGEWORTH_CLASSICAL"
};

static const unsigned nNames = sizeof(names)/sizeof(names[0]);

namespace npstat {
    EdgeworthSeriesMethod parseEdgeworthSeriesMethod(const char* methodName)
    {
        for (unsigned i=0; i<nNames; ++i)
            if (strcmp(names[i], methodName) == 0)
                return static_cast<EdgeworthSeriesMethod>(i);
        std::ostringstream os;
        os << "In npstat::parseEdgeworthSeriesMethod: invalid argument \""
           << methodName << "\". Must be one of "
           << validEdgeworthSeriesMethodNames() << '.';
        throw std::invalid_argument(os.str());
    }

    const char* edgeworthSeriesMethodName(EdgeworthSeriesMethod m)
    {
        const unsigned i = static_cast<unsigned>(m);
        assert(i < nNames);
        return names[i];
    }

    std::string validEdgeworthSeriesMethodNames()
    {
        std::ostringstream os;
        for (unsigned i=0; i<nNames; ++i)
        {
            if (i == 0)
                os << names[i];
            else if (i == nNames - 1)
                os << " or " << names[i];
            else
                os << ", " << names[i];
        }
        return os.str();
    }
}
