#ifndef NPSTAT_EDGEWORTHSERIESMETHOD_HH_
#define NPSTAT_EDGEWORTHSERIESMETHOD_HH_

/*!
// \file EdgeworthSeriesMethod.hh
//
// \brief Enumeration of methods used to create Edgeworth series
//
// Author: I. Volobouev
//
// June 2019
*/

#include <string>

namespace npstat {
    enum EdgeworthSeriesMethod
    {
        EDGEWORTH_SEVERINI = 0,  // The series will be w.r.t. normal
                                 // distribution with mean 0 and variance 1

        EDGEWORTH_CLASSICAL      // The series will be w.r.t. normal
                                 // distribution with mean and variance
                                 // defined by the first two cumulants
    };

    /** Enums corresponding to method names */
    EdgeworthSeriesMethod parseEdgeworthSeriesMethod(const char* methodName);

    /** Method names corresponding to enums */
    const char* edgeworthSeriesMethodName(EdgeworthSeriesMethod m);

    /** All valid method names for use in error messages, etc */
    std::string validEdgeworthSeriesMethodNames();
}

#endif // NPSTAT_EDGEWORTHSERIESMETHOD_HH_
