#ifndef NPSTAT_LIKELIHOODSTATISTICTYPE_HH_
#define NPSTAT_LIKELIHOODSTATISTICTYPE_HH_

/*!
// \file LikelihoodStatisticType.hh
//
// \brief Enumeration of types of likelihood-based signal detection statistics
//
// Author: I. Volobouev
//
// July 2019
*/

#include <string>

namespace npstat {
    enum LikelihoodStatisticType
    {
        L_STAT_R_W = 0, // Signed Wald, using Cramer-Rao sigma
        L_STAT_R_W2,    // Signed Wald, using observed information
        L_STAT_R_W3,    // Signed Wald, using delta-log-likelihood = -1/2 rule
        L_STAT_R_S,     // Signed score statistic
        L_STAT_R_LR     // Signed likelihood ratio
    };

     /** Enums corresponding to simplified statistic names */
    LikelihoodStatisticType parseLikelihoodStatisticType(const char* name);

    /** Statistic names corresponding to enums */
    const char* likelihoodStatisticName(LikelihoodStatisticType t);

    /**  All valid statistic names or use in error messages, etc */
    std::string validLikelihoodStatisticNames();
}

#endif // NPSTAT_LIKELIHOODSTATISTICTYPE_HH_
