#include <sstream>

#include "npstat/stat/SmoothGOFTest1D.hh"
#include "npstat/stat/Distributions1D.hh"

namespace npstat {
    SmoothGOFTest1D::SmoothGOFTest1D(const AbsDistribution1D& distro,
                                     const AbsClassicalOrthoPoly1D& poly,
                                     const unsigned i_maxdeg,
                                     const unsigned char* mask,
                                     const unsigned lenMask)
        : AbsUnbinnedGOFTest1D(distro),
          poly_(poly.clone()),
          aveBuf_(i_maxdeg+1U),
          statBuf_(i_maxdeg),
          mask_(i_maxdeg+1U, 0),
          maxdeg_(i_maxdeg),
          nUnmasked_(i_maxdeg)
    {
        if (lenMask)
        {
            assert(mask);
            if (lenMask != maxdeg_) throw std::invalid_argument(
                "In npstat::SmoothGOFTest1D constructor: "
                "incompatible mask length");
            nUnmasked_ = 0;
            mask_[0] = 1;
            for (unsigned i=0; i<lenMask; ++i)
            {
                mask_[i+1U] = mask[i];
                if (!mask[i])
                    ++nUnmasked_;
            }
            if (!nUnmasked_) throw std::invalid_argument(
                "In npstat::SmoothGOFTest1D constructor: "
                "all eigenvectors are masked");
        }
    }

    double SmoothGOFTest1D::analyticPValue(
        const double stat, unsigned long /* sz */) const
    {
        Gamma1D chisq(0.0, 2.0, nUnmasked_/2.0);
        return chisq.exceedance(stat);
    }

    std::string SmoothGOFTest1D::shortName() const
    {
        std::ostringstream os;
        os << "S_" << maxdeg_;
        if (maxdeg_ != nUnmasked_)
        {
            // Figure out the number of the last masked degree
            unsigned lastMasked = maxdeg_ + 1U;
            for (unsigned i=1U; i<=maxdeg_; ++i)
                if (mask_[i])
                    lastMasked = i;
            assert(lastMasked <= maxdeg_);

            // Add mask info
            os << '_';
            for (unsigned i=1; i<=lastMasked; ++i)
            {
                if (mask_[i])
                    os << '1';
                else
                    os << '0';
            }
        }
        return os.str();
    }

    double SmoothGOFTest1D::inverseExceedance(
        const double pvalue, const unsigned long sz,
        const double smin, const double smax) const
    {
        if (pvalue <= 0.0 || pvalue >= 1.0) throw std::invalid_argument(
            "In npstat::SmoothGOFTest1D::inverseExceedance: "
            "p-value argument must be inside (0, 1) interval");

        if (pvalue > 1.0e-10)
        {
            Gamma1D chisq(0.0, 2.0, nUnmasked_/2.0);
            return chisq.quantile(1.0 - pvalue);
        }
        else
            return AbsUnbinnedGOFTest1D::inverseExceedance(
                pvalue, sz, smin, smax);
    }
}
