#ifndef NPSTAT_UNBINNEDGOFTEST1DFACTORY_HH_
#define NPSTAT_UNBINNEDGOFTEST1DFACTORY_HH_

/*!
// \file UnbinnedGOFTest1DFactory.hh
//
// \brief Factories for 1-d GoF tests for use in interpretive language 
//        environments
//
// Author: I. Volobouev
//
// April 2022
*/

#include <map>
#include <string>

#include "npstat/stat/AbsDistribution1D.hh"
#include "npstat/stat/AbsUnbinnedGOFTest1D.hh"

namespace npstat {
    class AbsUnbinnedGOFTest1DFactory
    {
    public:
        inline virtual ~AbsUnbinnedGOFTest1DFactory() {}

        virtual AbsUnbinnedGOFTest1D* create(
            const AbsDistribution1D& null) const = 0;
    };

    template <typename T>
    class UnbinnedGOFTest1DFactory: public AbsUnbinnedGOFTest1DFactory
    {
    public:
        inline virtual ~UnbinnedGOFTest1DFactory() {}

        T* create(const AbsDistribution1D& null) const
        {
            return new T(null);
        }
    };

    /**
    // Use this factory to construct a 1-d GoF test from its null
    // distrubution.
    //
    // Not all GoF tests support this construction method. Iterate
    // over the keys in this map for the list of supported tests.
    */
    class DefaultUnbinnedGOFTest1DFactory :
        public std::map<std::string, AbsUnbinnedGOFTest1DFactory *>
    {
    public:
        DefaultUnbinnedGOFTest1DFactory();
        virtual ~DefaultUnbinnedGOFTest1DFactory();

        inline bool contains(const std::string& name) const
            {return this->find(name) != this->end();}

    private:
        DefaultUnbinnedGOFTest1DFactory(
            const DefaultUnbinnedGOFTest1DFactory&);
        DefaultUnbinnedGOFTest1DFactory& operator=(
            const DefaultUnbinnedGOFTest1DFactory&);
    };
}

#endif // NPSTAT_UNBINNEDGOFTEST1DFACTORY_HH_
