#ifndef NPSTAT_LIKELIHOODSTATISTICCUMULANTS_HH_
#define NPSTAT_LIKELIHOODSTATISTICCUMULANTS_HH_
/*!
// \file likelihoodStatisticCumulants.hh
//
// \brief Code for calculating cumulants of various signal detection
//        statistics for subsequent use in the Edgeworth series
//
// Author: I. Volobouev
//
// July 2019
*/

#include <vector>
#include <limits>

#include "npstat/stat/LikelihoodStatisticType.hh"

namespace npstat {
    /**
    // The meaning of the "order" argument is the same as in the
    // constructor of the EdgeworthSeries1D class.
    //
    // "N" is the sample size.
    //
    // The default values of the shape parameters play essenstially
    // the same role as "None" does in Python. A sufficient number
    // of shape parameters should be provided in order to generate
    // the cumulants up to the order requested.
    //
    // The returned vector of cumulants can be used as the corresponding
    // constructor argument of the EdgeworthSeries1D class.
    */
    std::vector<double> mixtureModelCumulants(
        LikelihoodStatisticType t,
        unsigned order, unsigned N,
        double gamma = std::numeric_limits<double>::max(),
        double rho = std::numeric_limits<double>::max(),
        double xi = std::numeric_limits<double>::max(),
        double zeta = std::numeric_limits<double>::max());

    /**
    // The meaning of the "order" argument is the same as in the
    // constructor of the EdgeworthSeries1D class.
    //
    // "mu" is the background rate.
    //
    // The default values of the shape parameters play essenstially
    // the same role as "None" does in Python. A sufficient number
    // of shape parameters should be provided in order to generate
    // the cumulants up to the order requested.
    //
    // The returned vector of cumulants can be used as the corresponding
    // constructor argument of the EdgeworthSeries1D class.
    */
    std::vector<double> poissonProcessCumulants(
        LikelihoodStatisticType t,
        unsigned order, double mu,
        double gamma = std::numeric_limits<double>::max(),
        double rho = std::numeric_limits<double>::max(),
        double xi = std::numeric_limits<double>::max(),
        double zeta = std::numeric_limits<double>::max());
}

#endif // NPSTAT_LIKELIHOODSTATISTICCUMULANTS_HH_
