#ifndef NPSTAT_SCANMULTIVARIATEDENSITYASWEIGHT_HH_
#define NPSTAT_SCANMULTIVARIATEDENSITYASWEIGHT_HH_

/*!
// \file scanMultivariateDensityAsWeight.hh
//
// \brief Scan a multivariate density for subsequent use as a tabulated weight
//
// Author: I. Volobouev
//
// September 2019
*/

#include <stdexcept>

#include "npstat/nm/ArrayND.hh"
#include "npstat/stat/AbsDistributionND.hh"

namespace npstat {
    /**
    // Utility function for scanning multivariate kernels and returning
    // ArrayND of minimal size which encloses the complete density support
    // region.
    //
    // Arguments are as follows:
    //
    // kernel        -- the density to scan
    //
    // maxDim        -- maximum number of steps to make in each dimension.
    //                  All values must be odd.
    //
    // bandwidthSet  -- density function bandwidth values in each dimension
    //
    // stepSize      -- scan step size in each dimension
    //
    // arrayLength   -- number of elements in each of the arrays
    //                  maxOctantDim, bandwidthSet, and stepSize
    */
    CPP11_auto_ptr<ArrayND<double> > scanMultivariateDensityAsWeight(
        const AbsDistributionND& kernel, const unsigned* maxDim,
        const double* bandwidthSet, const double* stepSize,
        unsigned arrayLength);

#ifdef SWIG
    inline ArrayND<double>* scanMultivariateDensityAsWeight2(
        const AbsDistributionND& kernel,
        const unsigned* maxDim, unsigned len1,
        const double* bandwidthSet, unsigned len2,
        const double* stepSize, unsigned len3)
    {
        const unsigned dim = kernel.dim();
        if (!dim || len1 != dim || len2 != dim || len3 != dim)
            throw std::invalid_argument(
                "In npstat::scanMultivariateDensityAsWeight2: "
                "incompatible argument dimensions");
        CPP11_auto_ptr<ArrayND<double> > p = scanMultivariateDensityAsWeight(
            kernel, maxDim, bandwidthSet, stepSize, dim);
        return p.release();
    }
#endif // SWIG
}

#endif // NPSTAT_SCANMULTIVARIATEDENSITYASWEIGHT_HH_
