#ifndef NPSTAT_DOUBLEHISTOBINSND_HH_
#define NPSTAT_DOUBLEHISTOBINSND_HH_

#include <cassert>
#include <stdexcept>

#include "npstat/wrap/NumpyTypecode.hh"
#include "npstat/stat/HistoND.hh"

namespace npstat {
    template <class Histo>
    PyObject* doubleHistoBinsND(const Histo& h, const bool doubleSize=false,
                                const bool mirror=true)
    {
        typedef typename Histo::value_type Numeric;

        const int dim = h.dim();
        if (dim <= 0) throw std::invalid_argument(
            "In npstat::doubleHistoBinsND: "
            "dimensionful input histogram expected");
        if (dim > static_cast<int>(CHAR_BIT*sizeof(unsigned long)))
            throw std::invalid_argument(
                "In npstat::doubleHistoBinsND: "
                "input histogram dimensionality is too high");
        npy_intp sh[CHAR_BIT*sizeof(unsigned long)];
        for (int idim=0; idim<dim; ++idim)
        {
            const unsigned nBins = h.axis(idim).nBins();
            sh[idim] = doubleSize ? 2U*nBins : nBins;
        }
        const int typenum = NumpyTypecode<double>::code;
        PyObject* arr = PyArray_SimpleNew(dim, &sh[0], typenum);
        if (arr)
        {
            double* a = (double*)PyArray_DATA((PyArrayObject*)arr);
            if (doubleSize)
            {
                ArrayShape ash(doubleShape(h.binContents().shape()));
                ArrayND<double> arr(externalMemArrayND(a, ash));
                assert(a == arr.data());
                if (mirror)
                    h.binContents().multiMirror(&arr);
                else
                {
                    arr.clear();
                    unsigned idx[CHAR_BIT*sizeof(unsigned long)];
                    const ArrayND<Numeric>& from(h.binContents());
                    const Numeric* fromData = from.data();
                    const unsigned long len = from.length();
                    for (unsigned long i=0; i<len; ++i)
                    {
                        from.convertLinearIndex(i, idx, dim);
                        arr.value(idx, dim) = static_cast<double>(fromData[i]);
                    }
                }
            }
            else
            {
                const Numeric* from = h.binContents().data();
                const unsigned long len = h.binContents().length();
                for (unsigned long i=0; i<len; ++i)
                    a[i] = static_cast<double>(from[i]);
            }
        }
        return arr;
    }
}

#endif // NPSTAT_DOUBLEHISTOBINSND_HH_
